/* SEPS525 */

#define INDEX  				0x00  // 
#define STATUS_RD			0x01  // The status read instruction reads the internal status of the SEPS525
#define OSC_CTL  			0x02  // Internal oscillator mode selection
#define IREF  				0x80  // Control reference voltage generation
#define CLOCK_DIV			0x03  // OSC frequency setting / Display frequency divide ration
#define REDUCE_CURRENT			0x04  // Reduced driving current
#define SOFT_RST  			0x05  // Soft reset active high
#define DISP_ON_OFF 			0x06  // Display ON/OFF
#define PRECHARGE_TIME_R  		0x08  // Precharge time R
#define PRECHARGE_TIME_G  		0x09  // Precharge time G
#define PRECHARGE_TIME_B  		0x0A  // Precharge time B
#define PRECHARGE_CURRENT_R  		0x0B  // Precharge current R
#define PRECHARGE_CURRENT_G  		0x0C  // Precharge current G
#define PRECHARGE_CURRENT_B  		0x0D  // Precharge current B
#define DRIVING_CURRENT_R		0x10  // DCR driving current R
#define DRIVING_CURRENT_G		0x11  // DCG driving current G
#define DRIVING_CURRENT_B		0x12  // DCB driving current B
#define DISPLAY_MODE_SET 		0x13  // RGB swap, Scan mode, Row scan shift directio, Column data shift direction
#define RGB_IF  			0x14  // RGB interface mode
#define RGB_POL 			0x15  // Vsync, dot clock polarity
#define MEMORY_WRITE_MODE		0x16  // automatic V+ H increment / decrement
#define MX1_ADDR  			0x17  // Memory Access Window Dimension
#define MX2_ADDR  			0x18  // Memory Access Window Dimension
#define MY1_ADDR  			0x19  // Memory Access Window Dimension
#define MY2_ADDR  			0x1A  // Memory Access Window Dimension
#define MEMORY_ACCESS_POINTER_X  	0x20  // horizontal start position of a window for write in memory
#define MEMORY_ACCESS_POINTER_Y  	0x21  // vertical start position of a window for write in memory
#define DDRAM_DATA_ACCESS_PORT  	0x22  // Internal DDRAM memory can be accessed
#define GRAY_SCALE_TABLE_INDEX  	0x50  // Index register for gamma table
#define GRAY_SCALE_TABLE_DATA 		0x51  // Data register for gamma table
#define DUTY  				0x28  // Display duty ratio 0x16~127)
#define DSL 				0x29  // Display start line 0x0~127-16)
#define D1_DDRAM_FAC  			0x2E  // First screen display horizontal address for display
#define D1_DDRAM_FAR  			0x2F  // First screen display vertical address for display
#define D2_DDRAM_SAC  			0x31  // Second screen display horizontal address for Display
#define D2_DDRAM_SAR  			0x32  // Second screen display vertical address for display
#define SCR1_FX1  			0x33  // The start/end address of active column outputs for the first screen
#define SCR1_FX2  			0x34  // The start/end address of active column outputs for the first screen
#define SCR1_FY1  			0x35  // the start/end address of active row outputs for the second screen
#define SCR1_FY2  			0x36  // The start/end address of active row outputs for the second screen
#define SCR2_SX1  			0x37  // 2nd Screen display size horizontal start
#define SCR2_SX2  			0x38  // 2nd Screen display size horizontal end
#define SCR2_SY1  			0x39  // 2nd Screen display size vertical start
#define SCR2_SY2  			0x3A  // 2nd Screen display size vertical end
#define SCREEN_SAVER_CONTEROL  		0x3B  // 1st, 2nd Screen screen saver control
#define SS_SLEEP_TIMER  		0x3C  // Screen saver sleep timer
#define SCREEN_SAVER_MODE  		0x3D  // Screen mode set
#define SS_SCR1_FU  			0x3E  // 1st Screen update timer based on frame sync
#define SS_SCR1_MXY 			0x3F  // 1st Screen Horizontal moving step
#define SS_SCR2_FU  			0x40  // 2nd Screen update timer based on frame sync
#define SS_SCR2_MXY 			0x41  // screen moving step
#define MOVING_DIRECTION  		0x42  // 2nd Screen moving direction
#define SS_SCR2_SX1  			0x47  // 2nd Screen image box horizontal start address
#define SS_SCR2_SX2  			0x48  // 2nd Screen image box horizontal end address
#define SS_SCR2_SY1  			0x49  // 2nd Screen image box vertical start address
#define SS_SCR2_SY2 			0x4A  // 2nd Screen image box vertical end address

#define OLED_Reg_COM   (*(volatile  u8*)0x60000000)

#define OLED_Reg_DAT   (*(volatile  u8*)0x60010000)
#define OLED_Reg_DAT16 (*(volatile u16*)0x60010000)
#define OLED_Reg_DAT32 (*(volatile u32*)0x60010000)

#define OLED_RST(x) GPIO_##x##Bits(GPIOB,GPIO_Pin_11)

void delay_ms(volatile u32 d)
{
 d*=63000;
 while(d--);
}

void OLED_Reset(void)
{
 OLED_RST(Reset);
 delay_ms(50);
 OLED_RST(Set);
 delay_ms(50);
}

void OLED_Send(u8 com,u8 dat)
{
 OLED_Reg_COM=com;
 OLED_Reg_DAT=dat;
}

void OLED_Init(void)
{
 //Reg:04h Action:Normal current and PS ON / Internal osc power off
 OLED_Send(REDUCE_CURRENT,0x03);
 delay_ms(50); //T0_05sec();
 //Reg:04h Action:Normal current and PS OFF
 OLED_Send(REDUCE_CURRENT,0x00);
 delay_ms(50); //T0_05sec();
 //Reg:3Bh Action:Screen Saver OFF
 OLED_Send(SCREEN_SAVER_CONTEROL,0x00);
 //Reg:02h Action:Export1 internal clock/OSC with external resister/Internal OSC ON
 OLED_Send(OSC_CTL,0x01);
 //Reg:03h Action:FR=120Hz DIV=1
 OLED_Send(CLOCK_DIV,0x90);
 //Reg:80h Action:PDAC OFF,DDAC OFF/Reference Volt.control with external resister
 OLED_Send(IREF,0x00);
 //Reg:08h Action:set color R precharge time
 OLED_Send(PRECHARGE_TIME_R,0x03);
 //Reg:09h Action:set color G precharge time
 OLED_Send(PRECHARGE_TIME_G,0x05);
 //Reg:0Ah Action:set color B precharge tiem
 OLED_Send(PRECHARGE_TIME_B,0x05);
 //Reg:0Bh Action:set color R precharge current
 OLED_Send(PRECHARGE_CURRENT_R,0x0a);
 //Reg:0Ch Action:set color G precharge current
 OLED_Send(PRECHARGE_CURRENT_G,0x0a);
 //Reg:0Dh Action:set color B precharge current
 OLED_Send(PRECHARGE_CURRENT_B,0x0a);
 //Reg:10h Action:set color R dot driving current
 OLED_Send(DRIVING_CURRENT_R,0x56);
 //Reg:11h Action:set color G dot driving current
 OLED_Send(DRIVING_CURRENT_G,0x4D);
 //Reg:12h Action:set color B dot driving current
 OLED_Send(DRIVING_CURRENT_B,0x46);
 //Reg:13h Action:Col D0 to D159/col normal display
 OLED_Send(DISPLAY_MODE_SET,0x00);
 //Reg:14h Action:MPU mode
 OLED_Send(RGB_IF,0x31);
 //Reg:16h Action: 8btis double transfer,65K support
 OLED_Send(MEMORY_WRITE_MODE, 0x76); //262k colors H+ V+ H..V..
 //Reg:17h Action:Memory addr.X start
 OLED_Send(MX1_ADDR,0x00);
 //Reg:18h Action:Memory addr.X end
 OLED_Send(MX2_ADDR,0x9f);
 //Reg:18h Action:Memory addr.Y start
 OLED_Send(MY1_ADDR,0x00);
 //Reg:1Ah Action:Memory addr.Y end
 OLED_Send(MY2_ADDR,0x7f);
 //Reg:20h Action:Memory X start addr.
 OLED_Send(MEMORY_ACCESS_POINTER_X,0x00);
 //Reg:21h Action:Memory Y start addr.
 OLED_Send(MEMORY_ACCESS_POINTER_Y,0x00);
 //Reg:28h Action:Display duty ratio
 OLED_Send(DUTY,0x7f);
 //Reg:29h Action:Display start line
 OLED_Send(DSL,0x00);
 //Reg:2Eh Action:Display First screen X start point
 OLED_Send(D1_DDRAM_FAC,0x00);
 //Reg:2Fh Action:Display First screen Y start point
 OLED_Send(D1_DDRAM_FAR,0x00);
 //Reg:31h Action:Display Second screen X start point
 OLED_Send(D2_DDRAM_SAR,0x00);
 //Reg:32h Action:Display Second screen Y start point
 OLED_Send(D2_DDRAM_SAR,0x00);
 //Reg:33h Action:Display size X start
 OLED_Send(SCR1_FX1,0x00);
 //Reg:34h Action:Display size X end
 OLED_Send(SCR1_FX2,0x9f);
 //Reg:35h Action:Display size Y start
 OLED_Send(SCR1_FY1,0x00);
 //Reg:36h Action:Display size Y end
 OLED_Send(SCR1_FY2,0x7f);
}

void OLED_Position(u8 x1,u8 x2,u8 y1,u8 y2)
{
 OLED_Send(MX1_ADDR,x1);
 OLED_Send(MX2_ADDR,x2);

 OLED_Send(MY1_ADDR,y1);
 OLED_Send(MY2_ADDR,y2);

 OLED_Send(MEMORY_ACCESS_POINTER_X,x1);
 OLED_Send(MEMORY_ACCESS_POINTER_Y,y1);

 OLED_Reg_COM=DDRAM_DATA_ACCESS_PORT;
}

void OLED_Clear(u32 color)
{
 register u32 i=160*128;
 OLED_Position(0,160-1,0,128-1);
 while(i--)
 {
  OLED_Reg_DAT=color>>16;
  OLED_Reg_DAT=color>>8;
  OLED_Reg_DAT=color;
 }
}

void OLED_ON(void)
{
 OLED_Send(DISP_ON_OFF,0x01);
}

void debug_indicator(u32 c)
{
 OLED_Clear(c);
 delay_ms(1000); //while(1);
}
